function w = feval(obj, varargin)
%FEVAL    Evaluate RKFUN at scalar or matrix arguments.
%
% Calling syntax: w = feval(obj, A);
%                 w = feval(obj, A, v);
%
% The rational function r represented by the obj variable can
% either be evaluated pointwise (if only A is provided), or as a
% matrix function r(A)*v if both A and v are provided.
%
% This function is used mainly by RKFIT which is described in
%
% [1] M. Berljafa and S. G{\"u}ttel. Generalized rational Krylov
%     decompositions with an application to rational approximation,
%     SIAM J. Matrix Anal. Appl., 36(2):894--916, 2015.

  switch nargin
   case 2
    AA = varargin{1};
    if isa(AA, 'rkfun')
      % Composition of two rkfuns.
      if length(AA.coeffs) > 2
        error(['FEVAL: Composition is currently only possible for' ...
               ' inner rkfuns of type at most [1,1].']);
      end
      % Get Moebius transform parameters (rho*z - eta)/(nu*z - mu).
      if length(AA.coeffs) == 1
        rho = 0; eta = AA.coeffs(1); mu = 1; nu = 0;
      elseif length(AA.coeffs) == 2
        rho = AA.coeffs(1)*AA.K(2, 1) - AA.coeffs(2)*AA.K(1, 1);
        eta = AA.coeffs(1)*AA.H(2, 1) - AA.coeffs(2)*AA.H(1, 1);
        mu  = AA.H(2, 1);
        nu  = AA.K(2, 1);
      end
      w = obj;
      w.K = (rho*obj.K - nu*obj.H);
      w.H = (eta*obj.K - mu*obj.H);
      % figure out new type
      [t1,t2] = type(AA);
      if t1 >= 2 && t2 >= 1,
           w.k = 0; % reset k for concatenation with non-degree 1-polt
      end
      return
    else
      % Numerical point-wise evaluation.
      [mA, nA] = size(AA);
      AA = AA(:);
      A.isreal = isreal(AA);
      A.multiply = @(rho, eta, x) rho*(AA.*x) - eta*x;
      A.solve    = @(nu,  mu,  x) (nu*AA-mu).\x;
      v = ones(mA*nA, 1);
    end
   case 3
    % Matrix function times a vector.
    A = varargin{1};
    v = varargin{2};
  end

  for j = 1:size(v, 2)
    if size(obj.K, 1) == 1
      % Constant function of type [0, 0].
      V = v(:, j);
    else
      V = rat_krylov(A, v(:,j), obj.K, obj.H);
    end    
    %res = norm(diag(AA)*V*obj.K - V*obj.H)/(norm(AA)*norm(V)*norm(obj.K)+norm(V)*norm(obj.H));
    %disp(['FEVAL: residual in rerunning the decomposition = ' num2str(res) ])
    w(:, j) = V*obj.coeffs;
    % Allocate more space (works for vpa/mp).
    if j == 1 && size(v, 2) > 1
      w(1, size(v, 2)) = 0;
    end
  end

  if nargin == 2, w = reshape(w, mA, nA); end

end % function
